/*
    libporty - async background processes
    Copyright (C) 2010..2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Author: libporty@igor2.repo.hu
*/

#ifdef WIN32
#include "win32_loader/loader2.c" /* it has to be included before windows.h */
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "abgproc.h"
#include <libporty/net/local.h>
#include <libporty/util/buf.h>
#include <libporty/util/argv.h>

const char *reasons[] = {"EXIT", "ABNR", "KILL"};
typedef enum ereason_e {
	BGPROC_EXITED,
	BGPROC_ABNORMAL, /* abnormal termination (segfault and things alike) */
	BGPROC_KILLED    /* was killed by term or kill */
} ereason_t;

typedef struct bgproc_status_s {
	ereason_t reason;
	int code;
} bgproc_status_t;


#ifndef WIN32
#define P_pid_t pid_t
#define P_NULL_PID 0
#include "abgproc_threaded.c"
#include "abgproc_unix.c"
#else
#define P_pid_t HANDLE
#define P_NULL_PID 0
#include "abgproc_threaded.c"
#include "abgproc_win32.c"
#endif

int P_net_async_bgproc(char *argv[], P_net_socket *ctrlfd, P_net_socket *fd_stdin, P_net_socket *fd_stdout)
{
	P_net_socket chfd_in, chfd_out;
	P_net_socket parent_fds[3];
	int          child_fds[3]  = {0, 1, 2};
	int ret;

	if ((fd_stdin == NULL) || (fd_stdout == NULL))
		return -1;

	ret = P_net_local_socket(fd_stdin, &chfd_in, P_net_nonblock_full);
	if (ret != 0)
		return ret;

	ret = P_net_local_socket(fd_stdout, &chfd_out, P_net_nonblock_full);
	if (ret != 0)
		return ret;

	parent_fds[0] = chfd_in;
	parent_fds[1] = chfd_out;
	parent_fds[2] = 2;

	ret = P_net_async_bgproc_inherit(argv, ctrlfd, parent_fds, child_fds, 3);
	close(chfd_in);
	close(chfd_out);
	return ret;
}
