/*  gentrex - generic tree expressions; simplistic lower-case-letter map
    Copyright (C) 2015  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/gentrex
    Author: gentrex@igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "helper_map_az.h"

#define get_map_idx(idx, name) \
	(idx = (*name - 'a'), ((idx >= 0) && (idx < sizeof(ctx->map)/sizeof(ctx->map[0])))) \

void gentrex_map_az_init(gentrex_map_az_t *user_ctx)
{
	memset(user_ctx, 0, sizeof(gentrex_map_az_t));
}

void gentrex_map_az_get(void *user_ctx, const char *name, gentrex_nodecomp_t *result)
{
	int idx;
	gentrex_map_az_t *ctx = (gentrex_map_az_t *)user_ctx;

	if (get_map_idx(idx, name))
		*result = ctx->map[idx];
}

void gentrex_map_az_set(void *user_ctx, const char *name, gentrex_nodecomp_t ndc)
{
	int idx;
	gentrex_map_az_t *ctx = (gentrex_map_az_t *)user_ctx;

	if (!get_map_idx(idx, name)) {
		fprintf(stderr, "Invalid map name: '%s' (has to be 'a'..'z')\n", name);
		exit(1);
	}
	ctx->map[idx] = ndc;
}


void *gentrex_map_az_get_node(void *user_ctx, const char *name)
{
	gentrex_nodecomp_t res;
	gentrex_map_az_get(user_ctx, name, &res);
	return res.straight;
}

void gentrex_map_az_set_node(void *user_ctx, const char *name, void *node)
{
	gentrex_nodecomp_t ndc;
	ndc.straight = node;
	ndc.inverted = NULL;
	gentrex_map_az_set(user_ctx, name, ndc);
}
