/*
    c99tree - input layer, uid->c99c_node_t cache
    Copyright (C) 2015  Tibor Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdlib.h>
#include <string.h>
#include "c99tree-cc.h"
#include "tree_load_uid.h"

/* temp hack until error handling is fixed */
#include <stdio.h>

struct c99c_uid2node_s {
	c99c_node_t **c99c_uid2node;
	int alloced;
};

void c99c_uid2node_save(c99c_uid2node_t *cache, c99c_node_t *n)
{
	if (n->uid >= cache->alloced) {
		int last_alloced;
		last_alloced = cache->alloced;
		cache->alloced = n->uid + 256;
		cache->c99c_uid2node = realloc(cache->c99c_uid2node, sizeof(c99c_node_t *) * cache->alloced);
		memset(&cache->c99c_uid2node[last_alloced], 0, sizeof(c99c_node_t *) * (cache->alloced - last_alloced));
	}
	cache->c99c_uid2node[n->uid] = n;
}

static void c99c_uid2node_resolve_(c99c_uid2node_t *cache, c99c_node_t *node)
{
	if ((node->loc.twin_parent_uid > 0) && (node->loc.twin_parent_uid < cache->alloced)) {
		if (cache->c99c_uid2node[node->loc.twin_parent_uid] == 0)
			fprintf(stderr, "c99tree load error: referencing non-existing twin-parent: %d\n", node->loc.twin_parent_uid);
		node->loc.twin_parent = cache->c99c_uid2node[node->loc.twin_parent_uid];
	}
	node->loc.twin_parent_uid_orig = node->loc.twin_parent_uid;
	node->loc.twin_parent_uid = -1;
}

void c99c_uid2node_resolve(c99c_uid2node_t *cache, c99c_node_t *root)
{
	for(; root != NULL; root = root->next) {
		c99c_uid2node_resolve_(cache, root);
		if (root->children != NULL)
			c99c_uid2node_resolve(cache, root->children);
	}
}

c99c_uid2node_t *c99c_uid2node_alloc(void)
{
	c99c_uid2node_t *cache;
	cache = calloc(sizeof(c99c_uid2node_t), 1);
	return cache;
}

void c99c_uid2node_free(c99c_uid2node_t *cache)
{
	if (cache->c99c_uid2node != NULL)
		free(cache->c99c_uid2node);
	free(cache);
}
