# plugin for c99tree

cmd_callmap()
{
	for n in "$@"
	do
		$TREECC  -o "$n.TREE"  "$n"
		awk -v "src=$n"   '
			/^ *CALL/ { print "call " $2 " from " fnc " in " src}
			/  FUNCDEF/ {print "func " $2 " in " src;  fnc=$2 }
		' < $n.TREE
	done
}

cmd_callgraph() {
	awk -v "op=$1" '
		BEGIN {
			colors=split("#FF0000 #00FF00 #0000FF #CC0000 #00CC00 #0000CC #AA0000 #00AA00 #0000AA  #990000 #009900 #000099", COLOR, " ")
			color=1;
		}

		function regsrc(src)
		{
			if (!(src in SRCS)) {
				SRCS[src] = COLOR[color]
				color++;
				if (color >= colors)
					color = 1;
			}
		}

		/^func / {
			callee=$2
			src=$4
			regsrc(src)
			SRC[callee] = src
			next
		}

		/^call / {
			callee=$2
			caller=$4
			src=$6
			regsrc(src)
			SRC[caller] = src
			CALL[caller] = CALL[caller] "|" callee
		}

		function id(s)
		{
			gsub("[^A-Za-z0-9]", "_", s)
			return s
		}

		function calllist() {
			for(caller in CALL)
				print "in " caller " " SRC[caller]

			for(caller in CALL)
				print "callees " caller " " ICALL[caller]

			for(caller in CALL)
				print "callers " caller " " CALLER[caller]
		}

		function graphviz() {
			print "digraph calls {"
			print " rankdir=\"LR\";"
			for(src in SRCS) {
				print "\tsubgraph " id(src) " {"
				print "\t\tlabel=\"" src "\""
				for(caller in SRC) {
					if (SRC[caller] == src)
						print "\t\t" id(caller) " [label=\"" caller " \\n " src "\" color=\"" SRCS[src] "\" style=filled]"
				}
				print "\t}"
			}

			for(caller in CALL) {
				v = split(ICALL[caller], C, ",")
				for(n = 1; n <= v; n++) {
					print "\t" id(caller) " -> " id(C[n])
				}
			}

			print "}"
		}

		END {
			for(caller in CALL) {
				delete FOUND
				v = split(CALL[caller], C, "|")
				for(n = 1; n <= v; n++) {
					if ((C[n] != "") && (C[n] in SRC) && (!(C[n] in FOUND))) {
						ICALL[caller] = ICALL[caller] "," C[n]
						CALLER[C[n]] = CALLER[C[n]] "," caller
						FOUND[C[n]]=1
					}
				}
				sub("^,", "", ICALL[caller])
			}
			for(caller in CALLER)
				sub("^,", "", CALLER[caller])

			if (op == "-g") graphviz();
			else if (op == "-l") calllist();
		}
	'
}
