#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <libporty/net/network.h>
#include <libporty/net_async/abgproc.h>

int test(char *cmd[], int killtick, int numticks)
{
	P_net_socket proc_stdin, proc_stdout, ctrl;
	char line[64];
	int len, n;
	struct P_pollfd fds[2];

	n = P_net_async_bgproc(cmd, &ctrl, &proc_stdin, &proc_stdout);
	if (n != 0) {
		fprintf(stderr, "Couldn't run bgprocess: %d\n", n);
		return 1;
	}

	fds[0].fd     = proc_stdout;
	fds[0].events = P_POLLIN;
	fds[1].fd     = ctrl;
	fds[1].events = P_POLLIN;
/*			fprintf(stderr, "stat: %d %d %d\n", proc_stdin, proc_stdout, ctrl);*/

	for(n = 0; n < numticks; n++) {
		if (n == killtick) {
			if (fds[1].events != 0) {
				printf("*killing process*\n");
				P_net_write(ctrl, "TERM\n", 5);
			}
			else
				printf("*not killing process - already dead*\n");
		}

		if (P_poll(fds, 2, 400) < 0) {
			fprintf(stderr, "ERROR: P_poll returned < 0\n");
			return 1;
		}
/*		printf("\n*tick*\n"); */

		if (fds[0].revents & P_POLLIN) {
			len = P_net_read(proc_stdout, line, sizeof(line)-1);
			if (len > 0) {
				line[len] = '\0';
				printf("READ: '%s'\n", line);
			}
			else if (len == 0) {
				printf("EOF from process stdio\n");
				fds[0].events = 0;
				fds[0].fd = -1;
			}
			else
				printf("error on process stdio\n");
		}

		if (fds[1].revents & P_POLLIN) {
			len = P_net_read(ctrl, line, sizeof(line)-1);
			if (len > 0) {
				line[len] = '\0';
				printf("CTRL: '%s'\n", line);
			}
			else if (len == 0) {
				printf("EOF from process ctrl\n");
				fds[1].events = 0;
				fds[1].fd = -1;
			}
			else
				printf("error on process ctrl\n");
		}
	}
	return 0;
}

int main()
{
	char *cmd1[] = {"echo", "hello", NULL};
#ifdef WIN32
	char *cmd2[] = {"pause", NULL};
#else
	char *cmd2[] = {"sleep", "10", NULL};
#endif

	srand(time(NULL));

	P_net_init();
	
	printf("------------------------\n");
	test(cmd1, -1, 8);
	printf("------------------------\n");
	test(cmd2, 5, 12);
	printf("------------------------\n");

	P_net_uninit();

	return 0;
}
