#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <libporty/net/network.h>
#include <libporty/net_async/adns.h>

#define safestr(s) ((s) == NULL ? "(null)" : (s))

static void request(P_net_socket fd, char *hostname, char *ip_addr, char *cookie)
{
	P_net_async_dns_req_t req;

	req.hostname = hostname;
	req.ip_addr = ip_addr;
	strncpy(req.cookie.s, cookie, sizeof(req.cookie.s));
	if (P_net_async_dns_req(fd, &req) != 0) {
		fprintf(stderr, "Failed to send dns request\n");
		abort();
	}
}

int main()
{
	P_net_socket dns;
	int len, n;
	struct P_pollfd fds[1];
	void *buf = NULL;
	P_net_async_dns_req_t ans;

	P_net_init();

	if (P_net_async_dns(&dns) != 0) {
		printf("Failed to create async dns\n");
		P_net_uninit();
		return 1;
	}

	request(dns, "localhost",   NULL,        "cookie1:localhost");
	request(dns, "nonexst.com", NULL,        "cookie2:fail");
	request(dns, NULL,          "127.0.0.1", "cookie3:127.0.0.1");

	fds[0].fd     = dns;
	fds[0].events = P_POLLIN;

	for(n = 0; n < 8; n++) {
		P_poll(fds, 1, 400);
		printf("\n*tick*\n");
		if (fds[0].revents != 0) {
			len = P_net_async_dns_read(&buf, fds[0].fd);
			if (len != 0) {
				printf("(EOF)\n");
				break;
			}

			while(1) {
				memset(&ans, 0, sizeof(ans));
				if (P_net_async_dns_parse(&buf, &ans) != 0)
					break;
				printf("Answer: [%d] %s %s (%s)\n", ans.outcome, safestr(ans.hostname), safestr(ans.ip_addr), ans.cookie.s);
				if (strcmp(ans.cookie.s, "cookie3:127.0.0.1") == 0) {
					P_net_async_dns_destroy(&buf, fds[0].fd); /* trigger an eof */
					break; /* make sure we don't read right after destroy */
				}
			}
		}
	}
	P_net_close(dns);

	P_net_uninit();
	return 0;
}
